#!/bin/bash

printUsage () {
  cat <<- EOT
	Usage: ${0##*/}: <VOLUME>
	
	  <VOLUME> Name of encrypted volume.
	
	EOT
}

queryDatabase () {
  sqlite3 /data/freenas-v1.db
}

getVolumeCount () {
  local VOL_NAME=$1

  queryDatabase <<- EOT
	SELECT COUNT(*)
	FROM storage_volume
	WHERE vol_encrypt > 0
	  AND vol_name    = '$VOL_NAME';
	EOT
}

volumeExist () {
  local VOL_NAME=$1
  local VOL_COUNT=$(getVolumeCount "$VOL_NAME")
  [[ 1 -eq "$VOL_COUNT" ]] && return 0 || return 1
}

getEncryptedDevices () {
  local VOL_NAME=$1

  queryDatabase <<- EOT
	SELECT d.disk_name,
	       d.disk_serial
	FROM storage_disk          d,
	     storage_encrypteddisk e,
	     storage_volume        v
	WHERE d.id       = e.encrypted_disk_id
	  AND v.id       = e.encrypted_volume_id
	  AND v.vol_name = '$VOL_NAME';
	EOT
}

main () {
  local VOL_NAME=$1
  local WORK_PATH="/tmp"
  local ARCHIVE="eli_metadata_${VOL_NAME}.tgz"
  local COL PART2 PART1 PART META_FILE META_FILE_LST

  if volumeExist "$VOL_NAME"
  then
    tar -xzf "${WORK_PATH}/${ARCHIVE}" -C "$WORK_PATH"

    while IFS=\| read -a COL
    do
      PART2="/dev/${COL[0]}p2"
      PART1="/dev/${COL[0]}p1"
      META_FILE="eli_${COL[0]}_${COL[1]}.meta"

      if   [[ -c "$PART2" ]]
      then PART=$PART2
      elif [[ -c "$PART1" ]]
      then PART=$PART1
      else continue
      fi

      #geli restore "${WORK_PATH}/${META_FILE}" "$PART" &
      grep -q 'GEOM::ELI' < "${WORK_PATH}/${META_FILE}" && echo Found something reasonable || echo Nothing reasonable found

      META_FILE_LST+="${WORK_PATH}/${META_FILE} "
    done <<- EOT
	$(getEncryptedDevices "$VOL_NAME")
	EOT

    wait
    rm $META_FILE_LST
  else
    printUsage
    return 1
  fi
}

main "$@"
exit $?
